//
//  main.c
//  Mandelbrot
//
//  Created by Nathan Romriell on 3/27/16.
//
//

#include <OpenGL/gl.h>
#include <GLUT/glut.h>
#include <stdio.h>

struct Type_rgb{
    float r;
    float g;
    float b;
};

static const int screenWidth = 800;
static const int screenHeight = 800;
static const int totalPixels = (screenWidth*screenHeight);
double zoom = 1.0f;
double moveX = 0;
double moveY = 0;
double cReal = 0.28;
double cImaginary = 0.005f;
float red = 0.01f;
float green = 0.01f;
float blue = 0.01f;
float zShift = 0;
float colorShift = 0.01f;
float movementChange = 0.01;

struct Type_rgb pixels[totalPixels], pattern[999];

void mandelbrotset()
{
    float x0, y0, x, y, xTemp;
    int iteration, max_iteration, loc=totalPixels;
    printf("Start");
    for(y0 = -1; y0 < 1; y0 = y0 + 0.0025)
        for(x0 = -1; x0 < 1; x0 = x0+0.0025){
            x = 0;
            y = 0;
            iteration = 0;
            max_iteration = 1000;
            
            while(((x*x) + (y*y) < (2*2)) && iteration < max_iteration){
                xTemp = (x*x) - (y*y) + x0;
                y = (2*x*y) + y0;
                
                x = xTemp;
                iteration = iteration + 1;
            }
            if (loc < 0) {
                break;
            }
            if (iteration >= 999){
                pixels[loc].r = 0;
                pixels[loc].g = 0;
                pixels[loc].b = 0;
            }else {
                pixels[loc].r = red*iteration;
                pixels[loc].g = green*iteration;
                pixels[loc].b = blue*iteration;
            }
            loc = loc - 1;
        }
}

void juliaSet()
{
    double newReal, newImaginary, oldReal, oldImaginary;
    int maxIterations = 300;
    
    
    for(int y = 0; y < screenHeight; y++)
        for(int x = 0; x < screenWidth; x++)
        {
            newReal = (x - screenWidth / 2) / (0.5 * zoom * screenWidth) + moveX;
            newImaginary = (y - screenHeight / 2) / (0.5 * zoom * screenHeight) + moveY;
            int i;
            for(i = 0; i < maxIterations; i++)
            {
                oldReal = newReal;
                oldImaginary = newImaginary;
                newReal = oldReal * oldReal - oldImaginary * oldImaginary + cReal;
                newImaginary = 2 * oldReal * oldImaginary + cImaginary;
                if((newReal * newReal + newImaginary * newImaginary) > 4) break;
            }
            float shiftZ = zShift*(20-i);
            int zShiftInt = (int)(shiftZ+0.5);
            int loc = (x+zShiftInt)+(y+zShiftInt)*screenWidth;
            if (loc > 0 && loc < totalPixels){
                pixels[loc].r = red*i;
                pixels[loc].g = green*i;
                pixels[loc].b = blue*i;
            }
        }
}

void redraw()
{
    for (int i = 0; i < totalPixels; i++){
        pixels[i].r = 0;
        pixels[i].g = 0;
        pixels[i].b = 0;
    }
    juliaSet();
    glutPostRedisplay();
}

void Init()
{
    glViewport(0,0,screenWidth, screenHeight);
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
   // gluOrtho2D(0,1440, 0, 841);
    
    int i;
    float r, g, b;
    
    for (i = 0; i < totalPixels; i++){
        pixels[i].r = 1;
        pixels[i].g = 1;
        pixels[i].b = 1;
    }
    i = 0;
    for (r = 0.1; r <= 0.9; r=r+0.1)
        for(g = 0.1; g <= 0.9; g = g+0.1)
            for(b = 0.1; b <= 0.9; b = b + 0.1){
                pattern[i].r = b;
                pattern[i].g = r;
                pattern[i].b = g;
                i++;
            }
    
    for (; i <= 999; i++){
        pattern[i].r = 1;
        pattern[i].g = 1;
        pattern[i].b = 1;
    }
    juliaSet();
}

void keyboardFunction(unsigned char Key, int x, int y)
{
    printf("Key Pressed:%c", Key);
    switch (Key) {
        case 'w':
            zoom = zoom+0.80f;
            break;
        case 's':
            zoom = zoom-0.80f;
            break;
        case 'a':
            cReal = cReal - 0.001f;
            break;
        case 'd':
            cReal = cReal + 0.001f;
            break;
        case 'e':
            cImaginary = cImaginary + 0.001f;
            break;
        case 'q':
            cImaginary = cImaginary - 0.001f;
            break;
        case ',':
            red = red-colorShift;
            break;
        case '.':
            red = red+colorShift;
            break;
        case ';':
            green = green-colorShift;
            break;
        case '\'':
            green = green+colorShift;
            break;
        case '[':
            blue = blue-colorShift;
            break;
        case ']':
            blue = blue+colorShift;
            break;
        case '=':
            zShift = zShift+0.10;
            break;
        case '-':
            zShift = zShift-0.10;
            break;
        case 'i':
            moveY = moveY-movementChange;
            break;
        case 'k':
            moveY = moveY+movementChange;
            break;
        case 'j':
            moveX = moveX+movementChange;
            break;
        case 'l':
            moveX = moveX-movementChange;
            break;
        default:
            break;
    }
    printf("Z Shift:%f", zShift);
    printf("Key Pressed:%c\n", Key);
    redraw();
}

void onDisplay()
{
    glClearColor(1,1,1,0);
    glClear(GL_COLOR_BUFFER_BIT);
    
    glDrawPixels(screenWidth, screenHeight, GL_RGB, GL_FLOAT, pixels);
    glutSwapBuffers();
}

int main(int argc, char** argv)
{
    glutInit(&argc, argv);
    glutInitWindowSize(screenWidth, screenHeight);
    glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH);
   // glutinitWindowPosition (100, 100);
    glutCreateWindow("Mandelbrotset");
    Init();
    
    glutDisplayFunc(onDisplay);
    glutKeyboardFunc(keyboardFunction);
    glutMainLoop();
    return 0;
}